@@ -14,7 +14,6 @@
 #include "nv-freebsd.h"
 
 static d_open_t  nvidia_dev_open;
-static void nvidia_dev_dtor(void *arg);
 static d_ioctl_t nvidia_dev_ioctl;
 static d_poll_t  nvidia_dev_poll;
 static d_mmap_single_t nvidia_dev_mmap_single;
@@ -67,7 +66,7 @@ static int nvidia_dev_open(
     return 0;
 }
 
-static void nvidia_dev_dtor(void *arg)
+void nvidia_dev_dtor(void *arg)
 {
     int status;
     struct nvidia_filep *filep = arg;
