@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2001-2016 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2001-2017 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -15,8 +15,27 @@
 
 #include <sys/consio.h>
 #include <sys/fbio.h>
+#include <sys/linker.h>
+#include <sys/timex.h>
 #include <dev/syscons/syscons.h>
 
+// This bootloader metadata interface was added to metadata.h in FreeBSD 10.1
+// with commit 3a7cd99956b1 and FreeBSD 11 with commit d2c99117cd3b.
+// https://github.com/freebsd/freebsd/blob/master/sys/x86/include/metadata.h
+#define NV_HAVE_EFI_FB (__FreeBSD_version >= 1001000)
+#define MODINFOMD_EFI_FB 0x1005
+struct efi_fb {
+    uint64_t fb_addr;
+    uint64_t fb_size;
+    uint32_t fb_height;
+    uint32_t fb_width;
+    uint32_t fb_stride;
+    uint32_t fb_mask_red;
+    uint32_t fb_mask_green;
+    uint32_t fb_mask_blue;
+    uint32_t fb_mask_reserved;
+};
+
 MALLOC_DEFINE(M_NVIDIA, "nvidia", "NVIDIA memory allocations");
 TASKQUEUE_DEFINE_THREAD(nvidia);
 
@@ -150,6 +169,18 @@ NV_STATUS NV_API_CALL os_get_current_time(
     return NV_OK;
 }
 
+#define NANOSECOND_PER_USEC 1000
+
+void NV_API_CALL os_get_current_tick(NvU64 *nsec)
+{
+    NvU32 sec, usec;
+
+    /* TODO: can we use getnanouptime() for this? */
+    (void) os_get_current_time(&sec, &usec);
+
+    *nsec = (NvU64)sec * NANOSECOND + (NvU64)usec * NANOSECOND_PER_USEC;
+}
+
 BOOL NV_API_CALL os_is_administrator(PHWINFO pDev)
 {
     return suser(CURTHREAD) ? FALSE : TRUE;
@@ -220,11 +251,6 @@ void* NV_API_CALL os_map_kernel_space(
         return NULL;
 #endif
 
-    if (start & PAGE_MASK)
-        return NULL;
-
-    size = NV_ALIGN_UP(size, PAGE_SIZE);
-
     switch (mode) {
         case NV_MEMORY_CACHED:
             map_mode = PAT_WRITE_BACK;
@@ -364,19 +390,18 @@ void NV_API_CALL os_dbg_breakpoint(void)
 #endif
 }
 
+#define MAX_ERROR_STRING 512
+static char nv_error_string[MAX_ERROR_STRING];
+
+/*
+ * The binary core of RM (nv-kernel.o) calls this:
+ */
 void NV_API_CALL out_string(const char *message)
 {
     printf("%s", message);
 }
 
-#define MAX_ERROR_STRING 512
-static char nv_error_string[MAX_ERROR_STRING];
-
-int NV_API_CALL nv_printf(
-    NvU32 debuglevel,
-    const char *format,
-    ...
-)
+int NV_API_CALL nv_printf(NvU32 debuglevel, const char *format, ...)
 {
     char *message = nv_error_string;
     va_list arglist;
@@ -392,12 +417,7 @@ int NV_API_CALL nv_printf(
     return chars_written;
 }
 
-NvS32 NV_API_CALL os_snprintf(
-    char *buf,
-    NvU32 size,
-    const char *fmt,
-    ...
-)
+NvS32 NV_API_CALL os_snprintf(char *buf, NvU32 size, const char *fmt, ...)
 {
     va_list arglist;
     int chars_written;
@@ -409,17 +429,9 @@ NvS32 NV_API_CALL os_snprintf(
     return chars_written;
 }
 
-void NV_API_CALL os_log_error(
-    NvU32 log_level,
-    NvU32 error_number,
-    const char *fmt,
-    va_list ap
-)
+void NV_API_CALL os_log_error(const char *fmt, va_list ap)
 {
-    int l;
-    sprintf(nv_error_string, "NVRM: ");
-    l = strlen(nv_error_string);
-    vsnprintf(nv_error_string + l, MAX_ERROR_STRING - l, fmt, ap);
+    vsnprintf(nv_error_string, MAX_ERROR_STRING, fmt, ap);
     printf("%s", nv_error_string);
 }
 
@@ -757,24 +769,57 @@ void NV_API_CALL os_get_screen_info(
     NvU16 *pFbPitch
 )
 {
-    const sc_softc_t *sc = sc_get_softc(0, SC_KERNEL_CONSOLE);
+#if NV_HAVE_EFI_FB
+    /*
+     * Look up EFI framebuffer information passed to the FreeBSD kernel by the
+     * bootloader.
+     *
+     * Adapted from a suggestion by Conrad Meyer <cem@freebsd.org>.
+     */
+    caddr_t kmdp = preload_search_by_type("elf kernel") ?:
+                   preload_search_by_type("elf64 kernel");
 
-    if (sc)
+    if (kmdp != NULL)
     {
-        const video_adapter_t *adp = sc->adp;
-
-        if (adp)
+        const struct efi_fb *efifb =
+            (const struct efi_fb *)preload_search_info(kmdp, MODINFO_METADATA |
+                                                             MODINFOMD_EFI_FB);
+        if (efifb != NULL)
         {
-            const struct video_info *vi = &adp->va_info;
+            int depth = fls(efifb->fb_mask_red | efifb->fb_mask_green |
+                            efifb->fb_mask_blue | efifb->fb_mask_reserved);
+            int bpp = roundup2(depth, NBBY);
 
-            if (vi && (vi->vi_flags & V_INFO_LINEAR))
+            *pPhysicalAddress = efifb->fb_addr;
+            *pFbWidth = efifb->fb_width;
+            *pFbHeight = efifb->fb_height;
+            *pFbDepth = depth;
+            /* fb_stride is in pixels. Convert to bytes */
+            *pFbPitch = efifb->fb_stride * (bpp / NBBY);
+            return;
+        }
+    }
+#endif
+    {
+        const sc_softc_t *sc = sc_get_softc(0, SC_KERNEL_CONSOLE);
+
+        if (sc)
+        {
+            const video_adapter_t *adp = sc->adp;
+
+            if (adp)
             {
-                *pPhysicalAddress = vi->vi_buffer;
-                *pFbWidth = vi->vi_width;
-                *pFbHeight = vi->vi_height;
-                *pFbDepth = vi->vi_depth;
-                *pFbPitch = adp->va_line_width;
-                return;
+                const struct video_info *vi = &adp->va_info;
+
+                if (vi && (vi->vi_flags & V_INFO_LINEAR))
+                {
+                    *pPhysicalAddress = vi->vi_buffer;
+                    *pFbWidth = vi->vi_width;
+                    *pFbHeight = vi->vi_height;
+                    *pFbDepth = vi->vi_depth;
+                    *pFbPitch = adp->va_line_width;
+                    return;
+                }
             }
         }
     }
@@ -973,3 +1018,82 @@ NV_STATUS NV_API_CALL os_call_vgpu_vfio(void *pvgpu_vfio_info, NvU32 cmd_type)
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NV_STATUS NV_API_CALL os_numa_memblock_size(NvU64 *memblock_size)
+{
+    return NV_ERR_NOT_SUPPORTED; 
+}
+
+NV_STATUS NV_API_CALL os_numa_online_memory
+(
+    NvS32  node_id, 
+    NvU64  region_gpu_addr, 
+    NvU64  region_gpu_size, 
+    NvU64  ats_base_addr, 
+    NvU64  memblock_size,
+    NvBool should_probe
+)
+{
+    return NV_ERR_NOT_SUPPORTED; 
+}
+
+void NV_API_CALL os_numa_offline_memory(NvS32 node_id)
+{
+}
+
+NV_STATUS NV_API_CALL os_alloc_pages_node
+(
+    NvS32  nid, 
+    NvU32  size, 
+    NvU32  flag, 
+    NvU64 *pAddress
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+NV_STATUS NV_API_CALL os_get_page
+(
+    NvU64 address
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+NV_STATUS NV_API_CALL  os_put_page
+(
+    NvU64 address
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+
+void NV_API_CALL os_free_pages_phys
+(
+    NvU64 address, 
+    NvU32 size
+)
+{
+}
+
+void NV_API_CALL os_unmap_kernel_numa
+(
+    void *address,
+    NvU64 size
+)
+{
+}
+
+NV_STATUS NV_API_CALL os_ipmi_connect(NvU32 devIndex, NvU8 myAddr, void **ppOsPriv)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+void NV_API_CALL os_ipmi_disconnect(void *pOsPriv)
+{
+}
+
+NV_STATUS NV_API_CALL os_ipmi_send_receive_cmd(void *pOsPriv, nvipmi_req_resp_t *pReq)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
