@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1999-2016 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1999-2017 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -26,6 +26,8 @@
 
 #include <stdarg.h>
 
+#include "nvipmi.h"
+
 /*
  * Define away Microsoft compiler extensions when possible
  */
@@ -76,6 +78,7 @@ NvU64       NV_API_CALL  os_get_num_phys_pages       (void);
 NV_STATUS   NV_API_CALL  os_alloc_mem                (void **, NvU64);
 void        NV_API_CALL  os_free_mem                 (void *);
 NV_STATUS   NV_API_CALL  os_get_current_time         (NvU32 *, NvU32 *);
+void        NV_API_CALL  os_get_current_tick         (NvU64 *);
 NV_STATUS   NV_API_CALL  os_delay                    (NvU32);
 NV_STATUS   NV_API_CALL  os_delay_us                 (NvU32);
 NvU64       NV_API_CALL  os_get_cpu_frequency        (void);
@@ -88,7 +91,7 @@ NvU32       NV_API_CALL  os_string_length            (const char *);
 NvU32       NV_API_CALL  os_strtoul                  (const char *, char **, NvU32);
 NvS32       NV_API_CALL  os_string_compare           (const char *, const char *);
 NvS32       NV_API_CALL  os_snprintf                 (char *, NvU32, const char *, ...);
-void        NV_API_CALL  os_log_error                (NvU32, NvU32, const char *, va_list);
+void        NV_API_CALL  os_log_error                (const char *, va_list);
 NvU8*       NV_API_CALL  os_mem_copy                 (NvU8 *, const NvU8 *, NvU32);
 NV_STATUS   NV_API_CALL  os_memcpy_from_user         (void *, const void *, NvU32);
 NV_STATUS   NV_API_CALL  os_memcpy_to_user           (void *, const void *, NvU32);
@@ -101,10 +104,11 @@ NV_STATUS   NV_API_CALL  os_pci_read_dword           (void *, NvU32, NvU32 *);
 NV_STATUS   NV_API_CALL  os_pci_write_byte           (void *, NvU32, NvU8);
 NV_STATUS   NV_API_CALL  os_pci_write_word           (void *, NvU32, NvU16);
 NV_STATUS   NV_API_CALL  os_pci_write_dword          (void *, NvU32, NvU32);
-NvBool      NV_API_CALL  os_remove_gpu_supported     (void);
-void        NV_API_CALL  os_remove_gpu               (NvU32, NvU8, NvU8);
+NvBool      NV_API_CALL  os_pci_remove_supported     (void);
+void        NV_API_CALL  os_pci_remove               (void *);
 void*       NV_API_CALL  os_map_kernel_space         (NvU64, NvU64, NvU32, NvU32);
 void        NV_API_CALL  os_unmap_kernel_space       (void *, NvU64);
+void        NV_API_CALL  os_unmap_kernel_numa        (void *, NvU64);
 void*       NV_API_CALL  os_map_user_space           (NvU64, NvU64, NvU32, NvU32, void **);
 void        NV_API_CALL  os_unmap_user_space         (void *, NvU64, void *);
 NV_STATUS   NV_API_CALL  os_flush_cpu_cache          (void);
@@ -168,7 +172,16 @@ NV_STATUS   NV_API_CALL  os_get_acpi_rsdp_from_uefi  (NvU32 *);
 void        NV_API_CALL  os_add_record_for_crashLog  (void *, NvU32);
 void        NV_API_CALL  os_delete_record_for_crashLog (void *);
 NV_STATUS   NV_API_CALL  os_call_vgpu_vfio           (void *, NvU32);
-
+NV_STATUS   NV_API_CALL  os_numa_memblock_size(NvU64 *);
+NV_STATUS   NV_API_CALL  os_numa_online_memory(NvS32, NvU64, NvU64, NvU64, NvU64, NvBool);
+void        NV_API_CALL  os_numa_offline_memory(NvS32);
+NV_STATUS   NV_API_CALL  os_alloc_pages_node         (NvS32, NvU32, NvU32, NvU64 *);
+NV_STATUS   NV_API_CALL  os_get_page                 (NvU64 address);
+NV_STATUS   NV_API_CALL  os_put_page                 (NvU64 address);
+void        NV_API_CALL  os_free_pages_phys          (NvU64, NvU32);
+NV_STATUS   NV_API_CALL  os_ipmi_connect             (NvU32 devIndex, NvU8 myAddr, void **ppOsPriv);
+void        NV_API_CALL  os_ipmi_disconnect          (void *pOsPriv);
+NV_STATUS   NV_API_CALL  os_ipmi_send_receive_cmd    (void *pOsPriv, nvipmi_req_resp_t *pReq);
 
 /*
  * ---------------------------------------------------------------------------
@@ -198,7 +211,6 @@ NV_STATUS   NV_API_CALL  os_call_vgpu_vfio           (void *, NvU32);
 
 void NV_API_CALL  out_string(const char *str);
 int  NV_API_CALL  nv_printf(NvU32 debuglevel, const char *printf_format, ...);
-void NV_API_CALL  nv_prints(NvU32 debuglevel, const char *string);
 
 /*
  * those NV_MEMORY_* and NV_PROTECT_* have dup defines in osCore.h
@@ -221,6 +233,7 @@ void NV_API_CALL  nv_prints(NvU32 debuglevel, const char *string);
 #define NV_MEMORY_WRITEBACK         5
 #define NV_MEMORY_DEFAULT           6
 #define NV_MEMORY_UNCACHED_WEAK     7
+#define NV_MEMORY_NUMA              8
 
 #define NV_PROTECT_READABLE   1
 #define NV_PROTECT_WRITEABLE  2
